// Copyright 1996-1998 Omni Development, Inc.  All rights reserved.
//
// This software may only be used and reproduced according to the
// terms in the file OmniSourceLicense.html, which should be
// distributed with this project and can also be found at
// http://www.omnigroup.com/DeveloperResources/OmniSourceLicense.html.

#import "OIAppKitImageProcessor.h"

#import <Foundation/Foundation.h>
#import <AppKit/AppKit.h>
#import <OmniBase/OmniBase.h>
#import <OmniFoundation/OmniFoundation.h>
#import <OWF/OWF.h>
#import <OIF/OIF.h>

RCS_ID("$Header: /Network/Developer/Source/CVS/OmniGroup/OmniComponents/ImageProcessors/appkit/OIAppKitImageProcessor.m,v 1.9 1998/12/10 21:26:49 kc Exp $")

@interface OIAppKitImageProcessor (Private)
+ (void)registerForFilteredImageTypes;
- (void)validateImage:(NSImage *)resultImage;
- (void)imageFromStream;
- (void)imageFromFile;
@end

@implementation OIAppKitImageProcessor

static NSMutableSet *unfilteredContentTypes;

+ (void)didLoad;
{
    [self registerForFilteredImageTypes];
}


// Init and dealloc

- initWithPipeline:(OWPipeline *)aPipeline;
{
    if (![super initWithPipeline:aPipeline])
        return nil;

    appkitProcessingDoneLock = [[NSLock alloc] init];

    return self;
}

- (void)dealloc;
{
    [appkitProcessingDoneLock release];
    [super dealloc];
}


// OWProcessor subclass

- (void)process;
{
    NSString *imageFilename;
    OWDataStream *imageStream;
    SEL imageSelector;

    [self addOmniImageToPipelineIfIncrementalDisplayIsDesired];

    // Wouldn't want to accidentally raise an old exception
    imagingException = nil;

    imageStream = [dataCursor dataStream];
    contentType = [dataCursor contentType];

    if ([unfilteredContentTypes containsObject:contentType]) {
	imageSelector = @selector(imageFromStream);
    } else {
	imageFilename = [[NSFileManager defaultManager] scratchFilenameNamed:[[[pipeline lastAddress] cacheKey] lastPathComponent]];
    
	if ([dataCursor contentType] != [OWContentType contentTypeForExtension:[imageFilename pathExtension]]) {
	    imageFilename = [imageFilename stringByAppendingPathExtension:[[dataCursor contentType] primaryExtension]];
	}
    
	[imageStream pipeToFilename:imageFilename contentCache:[pipeline contentCacheForLastAddress]];
	imageSelector = @selector(imageFromFile);
    }

    [imageStream waitForDataEnd];
    [appkitProcessingDoneLock lock];
    [self mainThreadPerformSelector:imageSelector];
    [appkitProcessingDoneLock lock];
    [appkitProcessingDoneLock unlock];

    // If processing in main thread generated an exception, raise it again in this thread.

    if (imagingException) {
	[imagingException autorelease];
	[imagingException raise];
    }
}

@end


@implementation OIAppKitImageProcessor (Private)

// register for any image type which NSImage can load
+ (void)registerForFilteredImageTypes;
{
    NSArray *imageFileTypes;
    unsigned int imageFileTypesIndex;
    OWContentType *omniImageContentType;
    NSArray *unfilteredImageFileTypes;
    unsigned int unfilteredImageFileTypesIndex;

    omniImageContentType = [OIImage contentType];
    imageFileTypes = [NSImage imageFileTypes];
    imageFileTypesIndex = [imageFileTypes count];

    while (imageFileTypesIndex--) {
        NSString *imageFileType;
	OWContentType *imageFileContentType;

        imageFileType = [imageFileTypes objectAtIndex:imageFileTypesIndex];
	imageFileContentType = [OWContentType contentTypeForExtension:imageFileType];

	if (!imageFileContentType) {
            NSMutableDictionary *descriptionDictionary;
	    NSString *contentTypeName;

	    descriptionDictionary = [[NSMutableDictionary alloc] init];
	    contentTypeName = [NSString stringWithFormat:@"image/x-%@", imageFileType];
            [descriptionDictionary setObject:[NSArray arrayWithObject:imageFileType] forKey:contentTypeName];
            [OWContentType registerItemName:@"extensions" bundle:nil description:descriptionDictionary];
            [descriptionDictionary release];

	    imageFileContentType = [OWContentType contentTypeForExtension:imageFileType];
	    if (!imageFileContentType) {
                NSLog(@"OIAppKitImageProcessor: Unable to create content type for extension '%@'", imageFileType);
		continue;
	    }
	}
        [self registerProcessorClass:self fromContentType:imageFileContentType toContentType:omniImageContentType cost:2.0];
    }

    // Keep a list of the ones that we can do without having to filter them.
    // These have to be done after the above dictionary registration since the content types might not exist before then.
    unfilteredImageFileTypes = [NSImage imageUnfilteredFileTypes];
    unfilteredImageFileTypesIndex = [unfilteredImageFileTypes count];
    unfilteredContentTypes = [[NSMutableSet alloc] initWithCapacity:unfilteredImageFileTypesIndex];

    while (unfilteredImageFileTypesIndex--)
        [unfilteredContentTypes addObject:[OWContentType contentTypeForExtension:[unfilteredImageFileTypes objectAtIndex:unfilteredImageFileTypesIndex]]];
}

- (void)validateImage:(NSImage *)resultImage;
{
    NS_DURING {
        [resultImage lockFocus];
	imagingException = nil;
        [resultImage unlockFocus];
    } NS_HANDLER {
        [resultImage unlockFocus];
        [resultImage release];
        resultImage = nil;

        // imagingException must be retained
        imagingException = [localException retain];
    } NS_ENDHANDLER;

    if (!imagingException) {
        [self setImageSize:[resultImage size]];
        [self updateImage:resultImage];
    }

    [appkitProcessingDoneLock unlock];
}

- (void)imageFromStream;
{
    NSImage *resultImage;
    NSData *streamData;

    streamData = [dataCursor readAllData];
    resultImage = [[NSImage alloc] initWithData:streamData];
    [resultImage setDataRetained:YES];
    [self validateImage:resultImage];
    [resultImage release];
}

- (void)imageFromFile;
{
    NSImage *resultImage;

    resultImage = [[NSImage alloc] initWithContentsOfFile:[[dataCursor dataStream] filename]];
    [resultImage setDataRetained:YES];
    [self validateImage:resultImage];
    [resultImage release];
}

@end
